"""
plots used witin vehicle class
"""
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import math
# TODO: create input for figure size - loads from "defaults" folder?
figure_size = (800,450)

wheel_colors = ['rgb(0, 0, 255)', 'rgb(0, 255, 0)', 'rgb(153, 0, 204)', 'rgb(255, 102, 0)']

def tire_details(veh):
    fig = make_subplots(rows = 2, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.11)
            # tire center plot depends on lock condition
    def setdash(x):
        if x == 0:
            return 'dash'
        elif x == 1:
            return 'solid'
    # slip angle
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lf_alpha * 180 / math.pi,
                        mode = 'lines',
                        name = 'LF',
                        line = dict(color = wheel_colors[0], width = 1,
                        dash = 'solid')),
                        row = 1, col = 1)
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rf_alpha * 180 / math.pi,
                        mode = 'lines',
                        name = 'RF',
                        line = dict(color = wheel_colors[1], width = 1,
                        dash = 'solid')),
                        row = 1, col = 1)
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rr_alpha * 180 / math.pi,
                        mode = 'lines',
                        name = 'RR',
                        line = dict(color = wheel_colors[2], width = 1,
                        dash = 'solid')),
                        row = 1, col = 1)
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lr_alpha * 180 / math.pi,
                        mode = 'lines',
                        name = 'LR',
                        line = dict(color = wheel_colors[3], width = 1,
                        dash = 'solid')),
                        row = 1, col = 1)
    # locked Status
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lf_lock,
                        showlegend=False,
                        mode = 'lines',
                        name = 'LF',
                        line = dict(color = wheel_colors[0], width = 1,
                        dash = 'solid')),
                        row = 2, col = 1)

    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rf_lock,
                        showlegend=False,
                        mode = 'lines',
                        name = 'RF',
                        line = dict(color = wheel_colors[1], width = 1,
                        dash = 'solid')),
                        row = 2, col = 1)

    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rr_lock,
                        showlegend=False,
                        mode = 'lines',
                        name = 'RR',
                        line = dict(color = wheel_colors[2], width = 1,
                        dash = 'solid')),
                        row = 2, col = 1)

    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lr_lock,
                        showlegend=False,
                        mode = 'lines',
                        name = 'LR',
                        line = dict(color = wheel_colors[3], width = 1,
                        dash = 'solid')),
                        row = 2, col = 1)

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.15, xanchor = 'left', x = 0.01),
        autosize = False,
        width = figure_size[0],
        height = figure_size[1],
        title = 'Tire Slip Angle & Saturation (locked) Status',
        template = 'plotly_white',
        font = dict(family = 'Arial', size = 16, color = 'black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1)
    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Locked Satus', row = 2, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Slip Angle (degrees)', row = 1, col = 1)

    fig.show()


def vertical_forces(veh):

    fig = go.Figure()

    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lf_fz,
                        mode = 'lines',
                        name = 'LF',
                        line = dict(color = wheel_colors[0], width = 1,
                        dash = 'solid')))
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rf_fz,
                        mode = 'lines',
                        name = 'RF',
                        line = dict(color = wheel_colors[1], width = 1,
                        dash = 'solid')))
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.rr_fz,
                        mode = 'lines',
                        name = 'RR',
                        line = dict(color = wheel_colors[2], width = 1,
                        dash = 'solid')))
    fig.add_trace(go.Scatter(x = veh.model.t, y = veh.model.lr_fz,
                        mode = 'lines',
                        name = 'LR',
                        line = dict(color = wheel_colors[3], width = 1,
                        dash = 'solid')))
    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.15, xanchor = 'left', x = 0.01),
        autosize = False,
        width = figure_size[0],
        height = figure_size[1],
        title = 'Vertical Tire Forces',
        template = 'plotly_white',
        font = dict(family = 'Arial', size = 16, color = 'black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Time (s)')
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Vertical Force (lb)')

    fig.show()



"""
functions for plotting data related to vehicle kinematics
"""
import plotly.graph_objects as go
from plotly.subplots import make_subplots

# TODO: create input for figure size - loads from "defaults" folder?
figure_size = (800,450)

"""
vehicle motion - single vehicle
"""
def plot_model(self):
    """
    takes SingleMotion class variable as input
    """

    fig = make_subplots(rows = 2, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.05)
    # velocity data
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Vx / 1.46667,
                            mode = 'lines',
                            name = 'X',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Vy / 1.46667,
                            mode = 'lines',
                            name = 'Y',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Vr / 1.46667,
                            mode = 'lines',
                            name = 'Resultant',
                            line = dict(color = 'rgb(0, 0, 0)', width = 2)),
                            row = 1, col = 1)
    # acceleration data
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Ax / 32.2,
                            showlegend=False,
                            mode = 'lines',
                            name = 'Ax',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Ay / 32.2,
                            showlegend=False,
                            mode = 'lines',
                            name = 'Ay',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.Ar / 32.2,
                            showlegend=False,
                            mode = 'lines',
                            name = 'Ar',
                            line = dict(color = 'rgb(0, 0, 0)', width = 2)),
                            row = 2, col = 1)

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize=False,
        width = 900,
        height = 500,
        title = f'{self.name} - Inertial Frame Velocity | Acceleration',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        yaxis = dict(showgrid = False),
        font = dict(family = 'Arial', size = 14, color = 'black'))

    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1)
    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Velocity (mph)', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Acceleration (g)', row = 2, col = 1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    fig = make_subplots(rows = 2, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.05)

    # velocity data in vehicle frame
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.vx / 1.46667,
                            mode = 'lines',
                            name = 'x',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.vy / 1.46667,
                            mode = 'lines',
                            name = 'y',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 1, col = 1)

    # acceleration data
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.ax / 32.2,
                            showlegend=False,
                            mode = 'lines',
                            name = 'ax',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.ay / 32.2,
                            showlegend=False,
                            mode = 'lines',
                            name = 'ay',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 2, col = 1)

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize=False,
        width = 900,
        height = 500,
        title = f'{self.name} - Vehicle Frame Velocity | Acceleration',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        yaxis = dict(showgrid = False),
        font = dict(family = 'Arial', size = 14, color = 'black'))

    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1)
    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Velocity (mph)', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Acceleration (g)', row = 2, col = 1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()


    fig = make_subplots(rows = 2, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.05)

    # forward tire forces
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.lf_fx,
                            mode = 'lines',
                            name = 'LF',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.rf_fx,
                            mode = 'lines',
                            name = 'RF',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.rr_fx,
                            mode = 'lines',
                            name = 'RR',
                            line = dict(color = 'rgb(153, 0, 204)', width = 2)),
                            row = 1, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.lr_fx,
                            mode = 'lines',
                            name = 'LR',
                            line = dict(color = 'rgb(255, 102, 0)', width = 2)),
                            row = 1, col = 1)

    # rightward tire forces
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.lf_fy,
                            showlegend=False,
                            mode = 'lines',
                            name = 'LF',
                            line = dict(color = 'rgb(0, 0, 255)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.rf_fy,
                            showlegend=False,
                            mode = 'lines',
                            name = 'RF',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.rr_fy,
                            showlegend=False,
                            mode = 'lines',
                            name = 'RR',
                            line = dict(color = 'rgb(153, 0, 204)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.lr_fy,
                            showlegend=False,
                            mode = 'lines',
                            name = 'LR',
                            line = dict(color = 'rgb(255, 102, 0)', width = 2)),
                            row = 2, col = 1)

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize=False,
        width = 900,
        height = 500,
        title = f'{self.name} - Tire Forces',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        yaxis = dict(showgrid = False),
        font = dict(family = 'Arial', size = 14, color = 'black'))

    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1)
    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Forward (lb)', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Rightward (lb)', row = 2, col = 1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()



    fig = make_subplots(rows = 2, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.07)

    # heading
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.theta_deg,
                            mode = 'lines',
                            name = 'heading',
                            line = dict(color = 'rgb(0, 0, 0)', width = 2)),
                            row = 1, col = 1)

    # rotational
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.oz_deg,
                            mode = 'lines',
                            name = 'omega',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            row = 2, col = 1)
    fig.add_trace(go.Scatter(x = self.model.t, y = self.model.alphaz_deg,
                            mode = 'lines',
                            name = 'alpha',
                            line = dict(color = 'rgb(204, 0, 0)', width = 2)),
                            row = 2, col = 1)


    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize=False,
        width = 900,
        height = 500,
        title = f'{self.name} Heading and Rotation',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        yaxis = dict(showgrid = False),
        font = dict(family = 'Arial', size = 14, color = 'black'))

    fig.update_xaxes(showgrid = False, title_text = 'Time (s)', row = 2, col = 1)
    fig.update_xaxes(showgrid = False, title_text = '', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Heading (deg)', row = 1, col = 1)
    fig.update_yaxes(showgrid = False, title_text = 'Omega (deg/s), Alpha (deg/s/s)', row = 2, col = 1)


    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()



"""
class for static impulse momentum model
"""

from .carpenter_momentum_calcs import impc
from .collision_plane import define_impact_plane, define_impact_edge

class IMPC():
    def __init__(name, veh1, veh2, user_sim_defaults = None):
        self.name = name
        self.type = 'impc_carpenter'             # class type for saving files
        self.veh1 = deepcopy(veh1)
        self.veh2 = deepcopy(veh2)

        print(f"Create impact point for {self.veh1.name} = striking vehicle")
        print("")
        self.veh1 = define_impact_plane(veh1)

        print(f"Create impacting edge for {self.veh2.name} = struck vehicle")
        print("")
        self.veh2 = define_impact_edge(veh2, iplane = False)

    # plot initial positions show data on chart
        # load defaults
        if user_sim_defaults:
            # TODO: create check for user sim_defaults_input
            self.sim_defaults = user_sim_defaults
        else:
            self.sim_defaults = {'vehicle_mu':0.3,
                                 'cor':0.2}
